
#ifndef PLS_SCHEDULER_IMPL_H
#define PLS_SCHEDULER_IMPL_H

namespace pls {
namespace internal {
namespace scheduling {

template<typename Function>
void scheduler::perform_work(Function work_section) {
  PROFILE_WORK_BLOCK("scheduler::perform_work")
  root_task<Function> master{work_section};

  // Push root task on stacks
  auto new_master = memory_->task_stack_for(0)->push(master);
  memory_->thread_state_for(0)->root_task_ = new_master;
  memory_->thread_state_for(0)->current_task_ = new_master;
  for (unsigned int i = 1; i < num_threads_; i++) {
    root_worker_task<Function> worker{new_master};
    auto new_worker = memory_->task_stack_for(0)->push(worker);
    memory_->thread_state_for(i)->root_task_ = new_worker;
    memory_->thread_state_for(i)->current_task_ = new_worker;
  }

  // Perform and wait for work
  sync_barrier_.wait(); // Trigger threads to wake up
  sync_barrier_.wait(); // Wait for threads to finish

  // Clean up stack
  memory_->task_stack_for(0)->pop<typeof(master)>();
  for (unsigned int i = 1; i < num_threads_; i++) {
    root_worker_task<Function> worker{new_master};
    memory_->task_stack_for(0)->pop<typeof(worker)>();
  }
}

}
}
}

#endif //PLS_SCHEDULER_IMPL_H
